#include "kScion.moc"
#include <kfiledialog.h>
#include <kapp.h>
#include <kmenubar.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kpopupmenu.h>
#include <qimage.h>
#include <qlabel.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qslider.h>
#include <qpushbutton.h>
#include <qgrid.h>
#include <kruler.h>
#include <qcheckbox.h>
#include <qlayout.h>
#include <qradiobutton.h>
#include <qvbuttongroup.h>

#include "scionlg3.h"
#include <stdio.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/fcntl.h>
#include <linux/ioctl.h>
#include <unistd.h>

float charToFloat(char c) {
  return ((int)c + 256) % 256;
}
char floatToChar(float f) {
  return (char)((int)(f + 128) % 256) - 128;
}

FGThread::FGThread(void* newData) {
  data = newData;
}
FGThread::FGThread() {
}

void FGThread::run() {

  KScion *d = (KScion *)data;

  
  do { 
    
      // Read in frame data
    if( d->sData->capturing ) {
      if( d->sData->captureSingleFrame ) {
      
        d->stopCapture();
      }
        // Read data from the frameGrabber
      ioctl(d->sData->deviceHandle,LG3_GRAB,NON_TRIG); // grab the next available frame
      lseek(d->sData->deviceHandle, 0, SEEK_SET);     
      read(d->sData->deviceHandle, d->sData->frameBuffer, 307208); 
      
        // mix into the average buffer and fill the display buffer
      float avgConst = d->sData->averageConst;
      int index = 0;
      for( int y=0; y<480; y++ ) {
        for( int x=0; x<640; x++ ) {
          float newVal = charToFloat(d->sData->frameBuffer[index]);
          if( d->sData->enableAverage ) {
            float oldVal = d->sData->averageBuffer[index];
            d->sData->averageBuffer[index] = (1.0-avgConst) * oldVal + avgConst * newVal;
          }
          else {
            d->sData->averageBuffer[index] = newVal;
          }
          if( d->sData->subtractGround ) {
            d->sData->displayBuffer[index] = d->sData->averageBuffer[index] - d->sData->groundBuffer[index];
          }
          else {
            d->sData->displayBuffer[index] = d->sData->averageBuffer[index];
          }
          index++;
        }
      }
    }
    
      // copy the display buffer into the display image
    float max, min;
    if( d->sData->subtractGround ) {
      max = 255.0;
      min = -255.0;
    }
    else {
      max = 255.0;
      min = 0.0;
    }
    float scale = 256.0 / (max - min);
    char l = floatToChar(0.0);
    char h = floatToChar(255.0);
    for( int i=0; i<307208; i++ ) {
      if( d->sData->frameBuffer[i] == l ) {
        *(d->sData->frameImage->scanLine(0) + i) = l;
      }
      else if( d->sData->frameBuffer[i] == h ) {
        *(d->sData->frameImage->scanLine(0) + i) = h;
      }
      else {
        *(d->sData->frameImage->scanLine(0) + i) = floatToChar((d->sData->displayBuffer[i]-min) * scale);
      }
    }
      
      // Do NOT operate on display buffer past this point!
    
    
    
      // Clear out old plot data
    d->sData->plotImage->fill(0);
    float plotMax = 0;
    for( int x=0; x<640; x++ ) {
      d->sData->plotData[x] = 0;
    }

      // Generate new plot data
    int numLines = d->sData->dataTop - d->sData->dataBottom + 1;
    float plotScale = 1.0 / (256.0 * (float)numLines);
    for( int x=0; x<640; x++ ) {
      for( int y =  d->sData->dataBottom; y<=  d->sData->dataTop; y++ ) {
        float val;
        val = d->sData->displayBuffer[x+y*640];
        d->sData->plotData[x] += val;
        if( val > plotMax ) {
          plotMax = val;
        }
      }
      d->sData->plotData[x] *= plotScale;
    }
    
      // Average plot if desired
    if( d->sData->plotAverageType > 0 ) {
      float plotFilter[640];
/*
      float gaussProfile[100];
      if( d->sData->plotAverageType == 3 ) {
        for( int i=0; i<100; i++ ) {
          GaussProfile[i] = ;
        }
      }*/
        // Calculate the average
      int w = d->sData->plotAverageWidth;
      float ifwDef = 1.0 / (2.0 * (float)w + 1.0);
      for( int i=0; i<640; i++ ) {
        float ifw = ifwDef;
        bool calcIfw = false;
        int startAvg = i - w;
        int stopAvg = i+w;
        if( startAvg<0 ) {
          startAvg = 0;
          calcIfw = true;
        }
        if( stopAvg > 639 ) {
          stopAvg = 639;
          calcIfw = true;
        }
        if( calcIfw ) {
          ifw = 1.0 / (stopAvg - startAvg + 1);
        }
        if( d->sData->plotAverageType == 1 ) {
          float avg = 0;
          for( int x = startAvg; x<=stopAvg; x++ ) {
            avg += d->sData->plotData[x];
          }
          avg *= ifw;
          plotFilter[i] = avg;
        }
        else if( d->sData->plotAverageType == 2 ) {
          float max = 0;
          float min = 1;
          for( int x = startAvg; x<=stopAvg; x++ ) {
            if( d->sData->plotData[x] > max ) {
              max = d->sData->plotData[x];
            }
            if( d->sData->plotData[x] < min ) {
              min = d->sData->plotData[x];
            }
          }
          float mid = min + (max-min) * 0.5; 
          plotFilter[i] = mid;
        }
/*
        else if( d->sData->plotAverageType == 3 ) {
          for( int x = startAvg; x<=stopAvg; x++ ) {
            plotFilter[x] += d->sData->plotData[i] * gaussProfile[abs(x-i)];
          }
        }
  */     
      }
      
        // Transfer to plot data
      for( int i=0; i<640; i++ ) {
        d->sData->plotData[i] = plotFilter[i];
      }
    }
      
      // Determine the range of plot data
    float bottom, top;
      // Set plot range
    if( d->sData->subtractGround ) {
//      scale = 0.5;
      bottom = -1.0;
      top = 1.0;
    }
    else {
//      scale = 1.0;
      bottom = 0.0;
      top = 1.0;
    }
    scale = 256.0 / (top-bottom);
      
      // Invert if desired
    if( d->sData->invertPlot ) {
      for( int i=0; i<640; i++ ) {
        d->sData->plotData[i] = top - (d->sData->plotData[i]-bottom);
      }
    }
      // Log if desired
    if( d->sData->logPlot ) {
      float lsh = log(.001);
      float lsc = - 1.0 / lsh;
      for( int i=0; i<640; i++ ) {
        d->sData->plotData[i] = (log(d->sData->plotData[i]+.001) - lsh) * lsc;
      }
    }
    
      // Draw plot
    for( int i=0; i<640; i++ ) {
      int yval = (int) (scale * (d->sData->plotData[i]-bottom));
      if( yval >= 255 ) {
        yval = 255;
      }        
      if( yval < 0 ) {
        yval = 0;
      }
      for( int y = 255-yval; y<256; y++ ) {
        *(d->sData->plotImage->scanLine(y) + i) = (char)150;
      }
    }
    
      // Add lines to images
    for( int x = 0; x<640; x++ ) {
      *(d->sData->frameImage->scanLine(d->sData->dataTop) + x) -= 64;
      *(d->sData->frameImage->scanLine(d->sData->dataBottom) + x) -= 64;
      *(d->sData->plotImage->scanLine((int)(d->sData->triggerData[x] * 255.0)) + x) = 255;
    }
    for( int y=0; y<255; y++ ) {
      *(d->sData->plotImage->scanLine(y) + d->sData->hPointer) = (char)255;
    }
    
      // Refresh images on screen
    QApplication::postEvent(d, new QCustomEvent( 1000 ));
    

    usleep(100000);
  } while( true ); //  while( d->sData->capturing );
 
}


KScion::KScion ( const char * name ) : KMainWindow ( 0L, name )
{

  sData = new scionData;
  sData->frameBuffer = new char[307208];
  sData->averageBuffer = new float[307208];
  sData->groundBuffer = new float[307208];
  sData->displayBuffer = new float[307208];
  for( int i=0; i<307208; i++ ) {
    sData->frameBuffer[i] = 0;
    sData->averageBuffer[i] = 0;
    sData->groundBuffer[i] = 0;
    sData->displayBuffer[i] = 0;
  }
  sData->triggerData = new float[640];
  sData->lookupTable = new char[256];
  sData->dacHigh = 140;
  sData->dacLow = 60;
  sData->frameImage = new QImage(640, 480, 8, 256);
  sData->frameFileImage = new QImage(640, 480, 8, 256);
  sData->framePixmap = new QPixmap(640, 480, 24);
  sData->plotImage = new QImage(640, 256, 8, 256);
  sData->plotPixmap = new QPixmap(640, 256, 24);
  sData->rulerImage = new QImage(640, 20, 8, 256);
  sData->rulerImage->fill(0);
  sData->rulerPixmap = new QPixmap(640, 20, 24);
  sData->rulerPixmap->convertFromImage(*(sData->rulerImage), Qt::ThresholdDither);
  sData->plotData = new float[640];
  sData->averageConst = 1.0;
  sData->dataTop = 50;
  sData->dataBottom = 50;
//  sData->triggerOffset = 128;
//  sData->triggerSlope = 0;
//  sData->triggerCurve = 0;
  sData->capturing = false;
  sData->plotAverageWidth = 0;
  sData->hPointer = 0;
  sData->hScale = 1.0;
  sData->invertPlot = false;
  sData->logPlot = false;
  sData->subtractGround = false;
  sData->plotAverageType = 0;
  for( int i=0; i<640; i++ ) {
    sData->triggerData[i] = 0;
    sData->plotData[i] = 0;
  }
  
    // Define the palate
  for( int i=0; i<256; i++ ) {
    sData->frameImage->setColor( i, qRgb(i,i,i) );
    sData->frameFileImage->setColor( i, qRgb(i,i,i) );
    sData->rulerImage->setColor( i, qRgb(i,i,i) );
    sData->plotImage->setColor( i, qRgb(i,i,i) );
  }
  sData->frameImage->setColor( 255, qRgb(255,100,100) );
  sData->frameImage->setColor( 0, qRgb(0,0,50) );
  sData->plotImage->setColor( 255, qRgb(0,100,0) );
  sData->rulerImage->setColor( 254, qRgb(0,100,0) );
  sData->rulerImage->setColor( 253, qRgb(100,100,0) );
  sData->rulerImage->setColor( 252, qRgb(100,0,0) );
  
  
  sData->deviceHandle = open("/dev/lg3-0",O_RDONLY);
  if( sData->deviceHandle == 0 ) {
    printf("Could not open device file /dev/lg3-0\n");
    exit(1);
  }
  for (int i=0; i<256; i++) {
    sData->lookupTable[i] = i;
  }
  ioctl(sData->deviceHandle,LG3_SET_LUT,sData->lookupTable);
  
    // TODO: Check to see if the device is opened correctly
        
  setCaption("kScion Framegrabber Interface");

  QPopupMenu *filemenu = new QPopupMenu;
  filemenu->insertItem( i18n( "&Start Capture" ), this, SLOT(startCapture()) );
  filemenu->insertItem( i18n( "S&top Capture" ), this, SLOT(stopCapture()) );
  filemenu->insertItem( i18n( "&Frame Capture" ), this, SLOT(frameCapture()) );
  filemenu->insertSeparator();
  filemenu->insertItem( i18n( "Open Image" ), this, SLOT(fileOpen()) );
  filemenu->insertItem( i18n( "Save Image" ), this, SLOT(fileSave()) );
  filemenu->insertItem( i18n( "Save Plot" ), this, SLOT(plotSave()) );
  filemenu->insertSeparator();
  filemenu->insertItem( i18n( "&Quit" ), kapp, SLOT(quit()) );
  QString about =
          i18n("kScion Framegrabber Interface 0.3\n\n"
                "Luke Campagnola\n"
                "lcampagn@mines.edu\n"
                "\n\n"
                "\n"
                "\n"
                "\n"
                "\n");
  QPopupMenu *helpmenu = helpMenu( about );
      
  
  KMenuBar *menu = menuBar();
  menu->insertItem( i18n( "&File" ), filemenu );
  menu->insertSeparator();
  menu->insertItem( i18n( "&Help" ), helpmenu );
  QGrid *grid = new QGrid(3, this);
  QHBox *sliderBox = new QHBox( grid );
    
  sData->sliderTop = new QSlider( 0, 479, 10, sData->dataTop, Qt::Vertical, sliderBox );
  sData->sliderTop->setFixedSize(15, 480);  

  sData->sliderBottom = new QSlider( 0, 479, 10, sData->dataBottom, Qt::Vertical, sliderBox );
  sData->sliderBottom->setFixedSize(15, 480);  
  
  sData->frameLabel = new QLabel(grid);
  sData->frameLabel->setFixedSize(640, 480);
  
  QVBox * frameVBox = new QVBox ( grid );

  sData->toggleCaptureButton = new QPushButton(frameVBox);
  sData->toggleCaptureButton->setText("Start Capture");
  QLabel *spacer1 = new QLabel(frameVBox);
  spacer1->setFixedHeight(15);
  QLabel *wlLabel = new QLabel(NULL, QString("White Level:"), frameVBox);
  sData->whiteLevel = new QSpinBox(0, 255, 1, frameVBox);
  QLabel *blLabel = new QLabel(NULL, QString("Black Level:"), frameVBox);
  sData->blackLevel = new QSpinBox(0, 255, 1, frameVBox);
  sData->whiteLevel->setValue(220);
  sData->blackLevel->setValue(40);
  setWhiteLevel();
  setBlackLevel();
  QLabel *spacer2 = new QLabel(frameVBox);
  spacer2->setFixedHeight(15);
//  QLabel *alLabel = new QLabel(NULL, QString("Average Constant:"), frameVBox);
  sData->enableAverageCheck = new QCheckBox("Time Average", frameVBox);
  sData->enableAverage = false;
  sData->averageLevel = new QSpinBox(1, 1000, 1, frameVBox);
  sData->averageLevel->setValue(1);
  sData->averageConstantLabel = new QLabel(NULL, QString("Avg Const: 1.0"), frameVBox);
  QLabel *spacer3 = new QLabel(frameVBox);
  spacer3->setFixedHeight(15);
  sData->subGroundCheck = new QCheckBox("Subtract Ground", frameVBox);
  sData->subGroundButton = new QPushButton(frameVBox);
  sData->subGroundButton->setText("Grab Ground Frame");
  
  sData->grabScaleButton = new QPushButton(grid);
  sData->grabScaleButton->setText("Grab Scale");
  sData->rulerLabel = new QLabel(grid);
  sData->rulerLabel->setFixedSize(640, 20);  
  sData->hScaleLabel = new QLabel(grid);
  QLabel *spacer4 = new QLabel(frameVBox);
  spacer4->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Ignored);
    
  QHBox *triggerHBox = new QHBox(grid);
//  sData->scaleTriggerCurveSlider = new QSlider( -128, 128, 10, sData->triggerCurve, Qt::Vertical, triggerHBox );
//  sData->scaleTriggerSlopeSlider = new QSlider( -128, 128, 10, sData->triggerSlope, Qt::Vertical, triggerHBox );
//  sData->scaleTriggerOffsetSlider = new QSlider( 0, 256, 10, sData->triggerOffset, Qt::Vertical, triggerHBox );
  sData->grabTriggerButton = new QPushButton("Grab\nTrigger\nProfile", triggerHBox);
  
  
  sData->plotLabel = new QLabel(grid);
  sData->plotLabel->setFixedSize(640, 256);  
  QVBox * plotVBox = new QVBox ( grid );
  
  sData->invertPlotCheck = new QCheckBox("Invert", plotVBox);
  sData->logPlotCheck = new QCheckBox("Log Scale", plotVBox);
  sData->avgRadioGroup = new QVButtonGroup("Average Type", plotVBox);
  sData->avgNoneRadio = new QRadioButton("None", sData->avgRadioGroup);
  sData->avgMeanRadio = new QRadioButton("Mean", sData->avgRadioGroup);
  sData->avgMedianRadio = new QRadioButton("Mid", sData->avgRadioGroup);
  //sData->avgGaussianRadio = new QRadioButton("Gaussian", sData->avgRadioGroup);
  QLabel* paLabel = new QLabel(NULL, QString("Plot Average Width:"), plotVBox);
  sData->plotAverageWidthSpinBox  = new QSpinBox(0, 100, 1, plotVBox);
  QLabel *spacer5 = new QLabel(plotVBox);
  spacer5->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Ignored);
  
  sData->findPeakButton = new QPushButton(grid);
  sData->findPeakButton->setText("Find Peak");
  sData->hPointerSlider = new QSlider(0, 640, 10, sData->hPointer, Qt::Horizontal, grid);  
  
  sData->hPointerLabel = new QLabel(grid);
  sData->hPointerLabel->setText("0.0");
  
  connect( sData->whiteLevel, SIGNAL(valueChanged(int)), this, SLOT(setWhiteLevel()) );    
  connect( sData->blackLevel, SIGNAL(valueChanged(int)), this, SLOT(setBlackLevel()) );    
  connect( sData->averageLevel, SIGNAL(valueChanged(int)), this, SLOT(setAverageConst()) );    
  connect( sData->sliderTop, SIGNAL(valueChanged(int)), this, SLOT(topSliderChanged()) );    
  connect( sData->sliderBottom, SIGNAL(valueChanged(int)), this, SLOT(bottomSliderChanged()) );    
//  connect( sData->scaleTriggerOffsetSlider, SIGNAL(valueChanged(int)), this, SLOT(tOffsetSliderChanged()) );    
//  connect( sData->scaleTriggerSlopeSlider, SIGNAL(valueChanged(int)), this, SLOT(tSlopeSliderChanged()) );    
//  connect( sData->scaleTriggerCurveSlider, SIGNAL(valueChanged(int)), this, SLOT(tCurveSliderChanged()) );    
  connect( sData->grabScaleButton, SIGNAL(clicked()), this, SLOT(grabScaleClicked()) );    
  connect( sData->findPeakButton, SIGNAL(clicked()), this, SLOT(findPeakClicked()) );
  connect( sData->hPointerSlider, SIGNAL(valueChanged(int)), this, SLOT(pointerSliderChanged()) );    
  connect( sData->plotAverageWidthSpinBox, SIGNAL(valueChanged(int)), this, SLOT(plotAverageChanged()) );    
  connect( sData->invertPlotCheck, SIGNAL(clicked()), this, SLOT(invertPlotChanged()) );    
  connect( sData->logPlotCheck, SIGNAL(clicked()), this, SLOT(logPlotChanged()) );    
  connect( sData->enableAverageCheck, SIGNAL(clicked()), this, SLOT(enableAverageChanged()) );    
  connect( sData->subGroundButton, SIGNAL(clicked()), this, SLOT(subGroundBtnClicked()) );    
  connect( sData->subGroundCheck, SIGNAL(clicked()), this, SLOT(subGroundChkChanged()) );    
  connect( sData->toggleCaptureButton, SIGNAL(clicked()), this, SLOT(toggleCaptureClicked()) );    
  connect( sData->grabTriggerButton, SIGNAL(clicked()), this, SLOT(grabTriggerClicked()) );    
  connect( sData->avgRadioGroup, SIGNAL(clicked(int)), this, SLOT(avgTypeClicked(int)) );    
  
  
  setCentralWidget(grid);
  
  fgThread = new FGThread((void*)this);  
  fgThread->start(); 
//  fgThread->wait();

} 

void KScion::fileOpen()
{
  QString filename = KFileDialog::getOpenFileName( QString::null, "*", this );
  if( ! filename.isEmpty() ) {
    sData->capturing = false;
    usleep(100000);  // Just make sure the video thread has stopped operating 
                    // on the image data
                    
    if( ! sData->frameFileImage->load(filename)) {
      QString msg = QString( i18n("Failed to open this file for some reason..") );
      KMessageBox::information( 0, msg, i18n( "Information" ), 
                  "fileOpenInformationDialog" );
    
    }

    // Dump the loaded frame immediately back into the filter buffer
    for( int i=0; i<307208; i++ ) {
      sData->averageBuffer[i] = *(sData->frameFileImage->scanLine(0) + i);
    }
          
  }
}


 
void KScion::fileSave()
{
  QString filename=KFileDialog::getSaveFileName( QString::null, "*.png", this );
  if( ! filename.isEmpty() ) {

      // Read the filter buffer into the image
    for( int i=0; i<307208; i++ ) {
      *(sData->frameFileImage->scanLine(0) + i) = floatToChar(sData->displayBuffer[i]);
    }
    
    
      // TODO: This should guess the file type!
    if( ! sData->frameFileImage->save(filename, "PNG", 100) ) {
      QString msg = QString( i18n("Failed to save this file for some reason..") );
      KMessageBox::information( 0, msg, i18n( "Information" ), 
                    "fileOpenInformationDialog" );
    }
  }   
}

void KScion::plotSave()
{
  QString filename=KFileDialog::getSaveFileName( "data.csv", "*.csv", this );
  if( ! filename.isEmpty() ) {
    FILE *DATA = fopen(filename, "w");
    if( DATA == NULL ) {
      QString msg = QString( i18n("Failed to save this file for some reason..") );
      KMessageBox::information( 0, msg, i18n( "Information" ), 
                    "fileOpenInformationDialog" );          
    }
    else {
      float pixelSize = sData->hScale / 640.0;
      for( int i=0; i<640; i++ ) {
        fprintf(DATA, "%f, %f\n", i*pixelSize, sData->plotData[i]);
      }
      fclose(DATA);
    }
  }
}

void KScion::startCapture()
{
  sData->capturing = true;
  sData->toggleCaptureButton->setText("Stop Capture");
}

void KScion::stopCapture()
{
  sData->capturing = false;
  sData->captureSingleFrame = false;
  sData->toggleCaptureButton->setText("Start Capture");
}

void KScion::frameCapture()
{
  sData->captureSingleFrame = true;  
  startCapture();
}


void KScion::setWhiteLevel() {
  ioctl(sData->deviceHandle,LG3_DAC_HIGH,sData->whiteLevel->value()); 
}
void KScion::setBlackLevel() {
  ioctl(sData->deviceHandle,LG3_DAC_LOW,sData->blackLevel->value());
}

void KScion::customEvent( QCustomEvent * e )
{
    if ( e->type() == 1000 ) {

      sData->framePixmap->convertFromImage(*(sData->frameImage), Qt::ThresholdDither);
      bitBlt(sData->frameLabel, 0, 0, sData->framePixmap, 0, 0, 640, 480, Qt::CopyROP);
      sData->plotPixmap->convertFromImage(*(sData->plotImage), Qt::ThresholdDither);
      bitBlt(sData->plotLabel, 0, 0, sData->plotPixmap, 0, 0, 640, 256, Qt::CopyROP);
      bitBlt(sData->rulerLabel, 0, 0, sData->rulerPixmap, 0, 0, 640, 480, Qt::CopyROP);


    }
}


void KScion::topSliderChanged() {
  if( sData->sliderTop->value() < sData->sliderBottom->value() ) {
    sData->sliderBottom->setValue(sData->sliderTop->value());
  }
  sData->dataTop = sData->sliderTop->value();
}


void KScion::bottomSliderChanged() {
  if( sData->sliderBottom->value() > sData->sliderTop->value() ) {
    sData->sliderTop->setValue(sData->sliderBottom->value());
  }
  sData->dataBottom = sData->sliderBottom->value();
}

/*
void KScion::tOffsetSliderChanged() {
  sData->triggerOffset = sData->scaleTriggerOffsetSlider->value();
  setCurveData();
}
void KScion::tSlopeSliderChanged() {
  sData->triggerSlope = sData->scaleTriggerSlopeSlider->value();
  setCurveData();
}
void KScion::tCurveSliderChanged() {
  sData->triggerCurve = sData->scaleTriggerCurveSlider->value();
  setCurveData();
}
*/

/*
void KScion::setCurveData() {

  for( int x=0; x<640; x++ ) {
    float py = (float)sData->triggerOffset/256.0 + (float)sData->triggerSlope/128000.0 * (float)(x-320) + (float)sData->triggerCurve/25600000.0 * (float)((x-320) * (x-320));
    if( py > 1.0 )
      py = 1.0;
    if( py < 0 )
      py = 0;
    sData->triggerData[x] = py;
  }
  
}
*/
void KScion::grabScaleClicked() {
  
    // Find trigger points
  int dataPts = 0;
  int triggers[640];
  for( int x = 1; x<640; x++ ) {
    float d = sData->plotData[x];
    float d1 = sData->plotData[x-1];
    float t = (1.0-sData->triggerData[x]);
    float t1 = (1.0-sData->triggerData[x-1]);
    if( d > t && d1 < t1 ) {
      triggers[dataPts] = x;
      dataPts++;
    }
  }

    // Find intervals
  float intervals[640];
  for( int i=0; i<dataPts-1; i++ ) {
    intervals[i] = (float)(triggers[i+1] - triggers[i]);
  }
  
    // Clean out data
  bool deviants[640];
  for( int i=0; i<640; i++ ) {
    deviants[i] = false;
  }
  float mean = 1;
  float stDev = 10;

  int num = dataPts - 1;
  int realNum = num;
  int maxPasses = 20;
  
  for( int c=0; c<maxPasses; c++ ) {
    
    mean = 0;
    for( int i=0; i<num; i++ ) {
      if( ! deviants[i] ) {
        mean += intervals[i];
      }
    }
    mean /= (float)realNum;
      
      // Find the standard deviation  
    stDev = 0;
    for( int i=1; i<num; i++ ) {
      if( ! deviants[i] ) {
        float dev = intervals[i] - mean;
        stDev += dev * dev;
      }
    }
    stDev = sqrt(stDev / (float)realNum);

    printf("i: %d   mean: %f   std: %f\n", c, mean, stDev);
    
//    if( stDev / mean < 0.5 ) {
    if( stDev < 0.7 ) {
      break;
    }
    
  
      // Clean out everything over stDev
    for( int i=0; i<num; i++ ) {
      if( ! deviants[i] ) {
        if( fabs(intervals[i] - mean) > stDev * 1.5 ) {
          deviants[i] = true;
          realNum--;
        } 
      }
    } 
    if( realNum < 1 ) {
      printf("couldn't form decent data from this set.\n");
      mean = 640;
      break;
    }
  }
    // Get rid of bad data
  if( mean <= 0 ) {
    mean = 640;
  }
  
  float avgTickDist = mean;  
  sData->hScale = 640.0 / avgTickDist;
    
    // Draw the ruler
  sData->rulerImage->fill(0);
  for( int i=0; i<dataPts; i++ ) {
    char color = 254;
    if( deviants[i] ) {
      color = 252;
    }
    for( int y = 15; y<20; y++ ) {
      *(sData->rulerImage->scanLine(y) + (int)triggers[i]) = color;
    }
  }
  
  int tick = 0;
  float tickPos = 0.0;
  while( tickPos<640.0 ) {
    int tickSize = 5;
    if( tick % 5 == 0 )
      tickSize = 10;
    if( tick % 10 == 0 )
      tickSize = 15;
    
    for( int y = 0; y<tickSize; y++ ) {
      *(sData->rulerImage->scanLine(y) + (int)tickPos) = (char)255;
    }
    tick++;
    tickPos += avgTickDist;
  }
  sData->rulerPixmap->convertFromImage(*(sData->rulerImage), Qt::ThresholdDither);
  char* buf = new char[20];
  sprintf(buf, "%0.4f, %0.4f", avgTickDist, stDev);
  sData->hScaleLabel->setText(buf);
  delete buf;
}

void KScion::setAverageConst() {
  sData->averageConst = 1.0 / (float)(sData->averageLevel->value());
  char* buf = new char[50];
  sprintf(buf, "Avg Const: %0.4f", sData->averageConst);
  sData->averageConstantLabel->setText(buf);
  delete buf;
}


void KScion::findPeakClicked() {
  // Decide which direction to look
  
  float dif = sData->plotData[sData->hPointer+1] - sData->plotData[sData->hPointer];
  int dir = 1;
  if( dif < 0 ) {
    dir = -1;
  }
  
  int index = sData->hPointer;
  float baseLevel = sData->plotData[sData->hPointer];
  float totalArea = 0;
  while( sData->plotData[index] >= baseLevel ) {
    totalArea += sData->plotData[index] - baseLevel;
    index++;
  }
  float halfArea = totalArea / 2;
  index = sData->hPointer;
  totalArea = 0;  
  while( totalArea < halfArea ) {
    totalArea += sData->plotData[index] - baseLevel;
    index++;
  }
  sData->hPointer = index - 1;
  sData->hPointerSlider->setValue(index-1);
    
}

void KScion::pointerSliderChanged() {
  sData->hPointer = sData->hPointerSlider->value();
  char *buf = new char(20);
  sprintf(buf, "%f", (float)sData->hPointer * sData->hScale / 640.0);
  sData->hPointerLabel->setText(buf);
  delete buf;
}

void KScion::plotAverageChanged() {
  sData->plotAverageWidth = sData->plotAverageWidthSpinBox->value();
}

void KScion::invertPlotChanged() {
  sData->invertPlot = sData->invertPlotCheck->isChecked();
}

void KScion::logPlotChanged() {
  sData->logPlot = sData->logPlotCheck->isChecked();
}


void KScion::enableAverageChanged() {
  sData->enableAverage = sData->enableAverageCheck->isChecked();
}

void KScion::subGroundBtnClicked() {
  for( int i=0; i<307208; i++ ) {
    sData->groundBuffer[i] = sData->averageBuffer[i];
  }
}

void KScion::subGroundChkChanged() {
  sData->subtractGround = sData->subGroundCheck->isChecked();
}

void KScion::toggleCaptureClicked() {
  if( sData->capturing ) {
    stopCapture();
  }
  else {
    startCapture();
  }
}

void KScion::grabTriggerClicked() {
  for( int i=0; i<640; i++ ) {
    sData->triggerData[i] = 1.0 - sData->plotData[i];
  }
}

void KScion::avgTypeClicked(int id) {
  sData->plotAverageType = id;
}


