#include <kmainwindow.h>
#include <qlabel.h>
#include <qslider.h>
#include <qspinbox.h>
#include <qthread.h>
#include <qpushbutton.h>
#include <qcheckbox.h>
#include <qradiobutton.h>
#include <qvbuttongroup.h>

class FGThread : public QThread {
  private:
    void* data;
      
  public:
    FGThread();
    FGThread(void* newData);
    virtual void run();

};

struct scionData {
  int deviceHandle;
  char *frameBuffer;
  float *averageBuffer, *groundBuffer, *displayBuffer;
  char *lookupTable;
  int dacHigh, dacLow;
  float averageConst;
  bool capturing, captureSingleFrame;
  QImage *frameImage;
  QImage *frameFileImage;
  QPixmap *framePixmap;
  QImage *plotImage;
  QPixmap *plotPixmap;
  QImage *rulerImage;
  QPixmap *rulerPixmap;
  QSpinBox* whiteLevel;
  QSpinBox* blackLevel;    
  QSpinBox* averageLevel;
  QLabel* averageConstantLabel;    
  QCheckBox *enableAverageCheck;
  bool enableAverage;
  QLabel* frameLabel;
  QLabel* rulerLabel;
  QLabel* plotLabel;
  QPushButton* grabScaleButton;
  QPushButton* toggleCaptureButton;
  QSlider* sliderTop;
  QSlider* sliderBottom;
  int dataTop, dataBottom;
//  QSlider* scaleTriggerOffsetSlider;
//  QSlider* scaleTriggerSlopeSlider;
//  QSlider* scaleTriggerCurveSlider;
//  int triggerOffset, triggerSlope, triggerCurve;
  QVButtonGroup *avgRadioGroup;
  QPushButton *grabTriggerButton;
  QRadioButton *avgNoneRadio, *avgMeanRadio, *avgMedianRadio, *avgGaussianRadio;
  float* triggerData;
  float *plotData;
  QCheckBox *plotGroundCheck;
  QPushButton *plotGroundButton;
  int hPointer;
  QSlider *hPointerSlider;
  QPushButton *findPeakButton;
  QLabel *hPointerLabel;
  QSpinBox *plotAverageWidthSpinBox;
  int plotAverageWidth;
  int plotAverageType;   // 0 = None, 1 = Mean, 2 = Median, 3 = Gaussian
  float hScale;
  QLabel *hScaleLabel;
  QCheckBox *invertPlotCheck;
  QCheckBox *logPlotCheck;
  bool invertPlot, logPlot;
  QCheckBox *subGroundCheck;
  QPushButton *subGroundButton;
  bool subtractGround;
};

class KScion : public KMainWindow
{
   Q_OBJECT
  
  public:
 
    scionData* sData;  
    KScion ( const char * name );
    QMutex mutex;
    void KScion::customEvent(QCustomEvent*);
    FGThread *fgThread;
     
  public slots:
 
    void fileOpen();
    void fileSave();
    void plotSave();
    void startCapture();
    void stopCapture();
    void frameCapture();
    void setWhiteLevel();
    void setBlackLevel();
    void topSliderChanged();
    void bottomSliderChanged();
//    void tOffsetSliderChanged();
//    void tSlopeSliderChanged();
//    void tCurveSliderChanged();
    void setAverageConst();
    void grabScaleClicked();
    void findPeakClicked();
    void pointerSliderChanged();
    void plotAverageChanged();
    void invertPlotChanged();
    void logPlotChanged();
    void enableAverageChanged();
    void subGroundBtnClicked();
    void subGroundChkChanged();
    void toggleCaptureClicked();
    void grabTriggerClicked();
    void avgTypeClicked(int);
        
  private:
    
//    void setCurveData();

};


